/*
 * Copyright (c) 2002 Valentin Nechayev <netch@netch.kiev.ua>
 * All rights reserved.
 * Based on libmilter server example (appropriate copyright is considered).
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: main.c,v 1.9 2003/04/28 18:36:06 netch Exp $
 */

#define CACHE_TTL 3600

/* Filter for verifying address of sender. */

#include <sys/types.h>
#include <sys/file.h>
#include <sys/queue.h>
#include <unistd.h>
#include <pthread.h>
#include <db.h>
#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>

#include "libmilter/mfapi.h"

typedef int bool;

#ifndef FALSE
# define FALSE	0
#endif /* ! FALSE*/
#ifndef TRUE
# define TRUE	1
#endif /* ! TRUE*/

#include "verify.h"

static void
setreply_allparts( SMFICTX* ctx, const struct address_entry* a )
{
	if( a->ace_cclass != 4 && a->ace_cclass != 5 )
		return;
	smfi_setreply(
		ctx,
		a->ace_cclass == 5 ? "554" : "454",
		a->ace_cclass == 5 ? "5.7.1" : "4.7.1",
		a->ace_msg );
}

sfsistat
mlfi_envfrom(ctx, envfrom)
	SMFICTX *ctx;
	char **envfrom;
{
	char address[MAXADDRLEN];
	int rc;
	struct address_entry* ace;
	size_t n0;
	const char* p0;
	const char* ef = envfrom[0];

	/*   :
	 * envfrom     ( milter?);
	 * envfrom  "<>" -  (  -
	 *   );
	 * envfrom   
	 */
	fprintf( stderr, "mlfi_envfrom(%p): called, envfrom[0]=%s\n",
		ctx, envfrom[0] );
	if( !ef || strlen( ef ) == 0 ||
		!strcmp( ef, "<>" ) )
	{
		return SMFIS_ACCEPT;
	}
	n0 = strlen( ef ); /* XXX should use strnlen() */
	if( n0 >= MAXADDRLEN-1 ) {
		smfi_setreply( ctx, "554", "5.7.0", "Too long address" );
		return SMFIS_REJECT;
	}
	p0 = ef;
	if( *p0 == '<' ) { ++p0; --n0; }
	if( n0 > 0 && p0[n0-1] == '>' ) { --n0; }
	memmove( address, p0, n0 ); address[n0] = 0;
	/* ,   '@', ģ  */
	p0 = strchr( address, '@' );
	if( !p0 )
		return SMFIS_REJECT; /*XXX*/
	/*     .
	 *  ,    .
	 */
	if( strspn( address,
		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
		"0123456789.-_@+/=" ) != n0 )
	{
		return SMFIS_ACCEPT;
	}

	rc = verify_address( address, &ace );
	fprintf( stderr, "smfi_envfrom(%p): verify_address() -> %d\n",
		ctx, rc );
	if( rc < 0 || !ace ) {
		if( ace ) {
			address_entry_free( ace );
			ace = NULL;
		}
		smfi_setreply( ctx, "451", "4.0.0", "Temporary failure" );
		return SMFIS_TEMPFAIL;
	}
	if( ace->ace_msg )
		setreply_allparts( ctx, ace );
	if( ace->ace_cclass == 2 )
		return SMFIS_CONTINUE;
	if( ace->ace_cclass == 4 )
		return SMFIS_TEMPFAIL;
	if( ace->ace_cclass == 5 )
		return SMFIS_REJECT;
	smfi_setreply( ctx, "451", "4.0.0", "Software failure" );
	return SMFIS_TEMPFAIL;
}

sfsistat
mlfi_envrcpt(ctx, envrcpt)
	SMFICTX *ctx;
	char **envrcpt;
{
	char address[MAXADDRLEN];
	int rc;
	struct address_entry* ace;
	size_t n0;
	const char* p0;
	const char* ef = envrcpt[0];

	/*   :
	 * envrcpt     ( milter?);
	 * envrcpt  "<>" -  (  -
	 *   );
	 * envrcpt   
	 */
	fprintf( stderr, "mlfi_envrcpt(%p): called, envrcpt[0]=%s\n",
		ctx, envrcpt[0] );
	if( !ef || strlen( ef ) == 0 ||
		!strcmp( ef, "<>" ) )
	{
		return SMFIS_ACCEPT;
	}
	n0 = strlen( ef ); /* XXX should use strnlen() */
	if( n0 >= MAXADDRLEN-1 ) {
		smfi_setreply( ctx, "554", "5.7.0", "Too long address" );
		return SMFIS_REJECT;
	}
	p0 = ef;
	if( *p0 == '<' ) { ++p0; --n0; }
	if( n0 > 0 && p0[n0-1] == '>' ) { --n0; }
	memmove( address, p0, n0 ); address[n0] = 0;
	/* ,   '@', ģ  */
	p0 = strchr( address, '@' );
	if( !p0 )
		return SMFIS_REJECT; /*XXX*/
	/*     .
	 *  ,    .
	 */
	if( strspn( address,
		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
		"0123456789.-_@+/=" ) != n0 )
	{
		return SMFIS_ACCEPT;
	}

	rc = verify_address( address, &ace );
	fprintf( stderr, "smfi_envrcpt(%p): verify_address() -> %d\n",
		ctx, rc );
	if( rc < 0 || !ace ) {
		if( ace ) {
			address_entry_free( ace );
			ace = NULL;
		}
		smfi_setreply( ctx, "451", "4.0.0", "Temporary failure" );
		return SMFIS_TEMPFAIL;
	}
	if( ace->ace_msg )
		setreply_allparts( ctx, ace );
	if( ace->ace_cclass == 2 )
		return SMFIS_CONTINUE;
	if( ace->ace_cclass == 4 )
		return SMFIS_TEMPFAIL;
	if( ace->ace_cclass == 5 )
		return SMFIS_REJECT;
	smfi_setreply( ctx, "451", "4.0.0", "Software failure" );
	return SMFIS_TEMPFAIL;
}

struct smfiDesc smfilter =
{
	"VerifySender",	/* filter name */
	SMFI_VERSION,	/* version code -- do not change */
	0,		/* flags */
	NULL,		/* connection info filter */
	NULL,		/* SMTP HELO command filter */
	mlfi_envfrom,	/* envelope sender filter */
	mlfi_envrcpt,	/* envelope recipient filter */
	NULL,		/* header filter */
	NULL,		/* end of header */
	NULL,		/* body block filter */
	NULL,		/* end of message */
	NULL,		/* message aborted */
	NULL		/* connection cleanup */
};

int
main(argc, argv)
	int argc;
	char *argv[];
{
	int c;
	const char *args = "p:";
	extern int DL( int, int );

	puts( "verify_sender 0.1.2-current" );
	/* Process command line options */
	while ((c = getopt(argc, argv, args)) != -1)
	{
		switch (c)
		{
		case 'p':
			if (optarg == NULL || *optarg == '\0')
			{
				(void) fprintf(stderr, "Illegal conn: %s\n",
					       optarg);
				exit(EX_USAGE);
			}
			(void) smfi_setconn(optarg);
		break;

		}
	}
	verify_init();
	if (smfi_register(smfilter) == MI_FAILURE)
	{
		fprintf(stderr, "smfi_register failed\n");
		exit(EX_UNAVAILABLE);
	}
	;{
		extern void DLset( int, int );
		int i;
		for( i = 1; i <= 20; ++i )
			DLset( i, 30 );
	}
	if( DL( 1, 10 ) ) {
		fprintf( stderr, "verify_sender[%d]: started\n",
			(int) getpid() );
	}
	//smfi_settimeout(0);

	return smfi_main();
}

